/*
    quicksort.c - linear tree quicksort for legacy array.c
    
    Copyright (C) 2009  Laurent Poirrier
    This file is part of the lin-kd-tree library.

    The lin-kd-tree library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, version 3 of the License.

    The lin-kd-tree library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <string.h>
#include "kd-tree.h"
#include "array.h"
#include "quicksort.h"

// ********************************************************************
//
// ********************************************************************
static void kdtree_node_swap(kdtree_object *tree, int i, int j)
{
	kdtree_object tmp;
	
	//if (i == j) return;
	
	memcpy(&tmp, &tree[i], sizeof(tmp));
	memcpy(&tree[i], &tree[j], sizeof(tmp));
	memcpy(&tree[j], &tmp, sizeof(tmp));
}

// ********************************************************************
//
// ********************************************************************
static int kdtree_qs_partition(kdtree_object *tree, int left, int right, int pivot)
{
	kdtree_node *pivot_value;
	int r, i;
	
	// place pivot at right end, and store its value
	kdtree_node_swap(tree, pivot, right);
	pivot_value = &(tree[right].path);

	r = left;
	
	for (i = left; i < right; i++) {
		// accumulate values least than pivot_value at left end
		// (before index r)
		if (KDTREE_BEFORE(tree[i].path, *pivot_value)) {
			kdtree_node_swap(tree, i, r);
			r++;
		}
	}
	kdtree_node_swap(tree, r, right);
	return(r);
}

// ********************************************************************
//
// ********************************************************************
void kdtree_quicksort(kdtree_object *tree, int left, int right)
{
	int pivot1;
	
	if (right <= left) return;
	
	pivot1 = kdtree_qs_partition(tree, left, right, (left + right) >> 1);
	kdtree_quicksort(tree, left, pivot1 - 1);
	kdtree_quicksort(tree, pivot1 + 1, right);
	// a loop should be used as terminal recursion
}



